@@ -240,7 +240,7 @@ class BucketController(Controller):
             if status == 401:
                 return get_err_response('AccessDenied')
             elif status == 404:
-                return get_err_response('InvalidBucketName')
+                return get_err_response('NoSuchBucket')
             else:
                 return get_err_response('InvalidURI')
 
@@ -311,7 +311,7 @@ class BucketController(Controller):
             if status == 401:
                 return get_err_response('AccessDenied')
             elif status == 404:
-                return get_err_response('InvalidBucketName')
+                return get_err_response('NoSuchBucket')
             elif status == 409:
                 return get_err_response('BucketNotEmpty')
             else:
@@ -354,7 +354,8 @@ class ObjectController(Controller):
                 if _key.startswith('x-object-meta-'):
                     new_hdrs['x-amz-meta-' + key[14:]] = val
                 elif _key in ('content-length', 'content-type',
-                             'content-encoding', 'etag', 'last-modified'):
+                              'content-range', 'content-encoding',
+                              'etag', 'last-modified'):
                     new_hdrs[key] = val
             return Response(status=status, headers=new_hdrs, app_iter=app_iter)
         elif status == 401:
@@ -397,7 +398,7 @@ class ObjectController(Controller):
             if status == 401:
                 return get_err_response('AccessDenied')
             elif status == 404:
-                return get_err_response('InvalidBucketName')
+                return get_err_response('NoSuchBucket')
             else:
                 return get_err_response('InvalidURI')
 
