@@ -182,11 +182,30 @@ class FakeApp(object):
             return self.listing(env, start_response,
                                 {'x-container-read': '.r:*',
                                  'x-container-meta-web-listings': 'f'})
+        elif env['PATH_INFO'] in ('/v1/a/c8', '/v1/a/c8/'):
+            return self.listing(env, start_response,
+                          {'x-container-read': '.r:*',
+                           'x-container-meta-web-error': 'error.html',
+                           'x-container-meta-web-listings': 't',
+                           'x-container-meta-web-listings-css': \
+                               'http://localhost/stylesheets/listing.css'})
+        elif env['PATH_INFO'] == '/v1/a/c8/subdir/':
+            return Response(status='404 Not Found')(env, start_response)
+        elif env['PATH_INFO'] in ('/v1/a/c9', '/v1/a/c9/'):
+            return self.listing(env, start_response,
+                          {'x-container-read': '.r:*',
+                           'x-container-meta-web-error': 'error.html',
+                           'x-container-meta-web-listings': 't',
+                           'x-container-meta-web-listings-css': \
+                               '/absolute/listing.css'})
+        elif env['PATH_INFO'] == '/v1/a/c9/subdir/':
+            return Response(status='404 Not Found')(env, start_response)
         else:
             raise Exception('Unknown path %r' % env['PATH_INFO'])
 
     def listing(self, env, start_response, headers):
-        if env['PATH_INFO'] in ('/v1/a/c3', '/v1/a/c4') and \
+        if env['PATH_INFO'] in ('/v1/a/c3', '/v1/a/c4', '/v1/a/c8', \
+               '/v1/a/c9') and \
                env['QUERY_STRING'] == 'delimiter=/&format=json&prefix=subdir/':
             headers.update({'X-Container-Object-Count': '11',
                             'X-Container-Bytes-Used': '73741',
@@ -269,7 +288,7 @@ class FakeApp(object):
                   "hash":"c85c1dcd19cf5cbac84e6043c31bb63e", "bytes":20,
                   "content_type":"text/plain",
                   "last_modified":"2011-03-24T04:27:52.734140"},
-                 {"name":"subdir/omgomg.txt",
+                 {"name":"subdir/\u2603.txt",
                   "hash":"7337d028c093130898d937c319cc9865", "bytes":72981,
                   "content_type":"text/plain",
                   "last_modified":"2011-03-24T04:27:52.735460"},
@@ -290,7 +309,7 @@ class FakeApp(object):
                             'Content-Type': 'text/plain; charset=utf-8'})
             body = '\n'.join(['401error.html', '404error.html', 'index.html',
                               'listing.css', 'one.txt', 'subdir/1.txt',
-                              'subdir/2.txt', 'subdir/omgomg.txt', 'subdir2',
+                              'subdir/2.txt', u'subdir/\u2603.txt', 'subdir2',
                               'subdir3/subsubdir/index.html', 'two.txt'])
         return Response(status='200 Ok', headers=headers,
                         body=body)(env, start_response)
@@ -336,6 +355,16 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank('/v1/a/c1').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 401)
 
+    def test_container1_web_mode_explicitly_off(self):
+        resp = Request.blank('/v1/a/c1',
+            headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 401)
+
+    def test_container1_web_mode_explicitly_on(self):
+        resp = Request.blank('/v1/a/c1',
+            headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
     def test_container2(self):
         resp = Request.blank('/v1/a/c2').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 200)
@@ -343,6 +372,19 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(len(resp.body.split('\n')),
                           int(resp.headers['x-container-object-count']))
 
+    def test_container2_web_mode_explicitly_off(self):
+        resp = Request.blank('/v1/a/c2',
+            headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(resp.content_type, 'text/plain')
+        self.assertEquals(len(resp.body.split('\n')),
+                          int(resp.headers['x-container-object-count']))
+
+    def test_container2_web_mode_explicitly_on(self):
+        resp = Request.blank('/v1/a/c2',
+            headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
     def test_container2onetxt(self):
         resp = Request.blank(
                 '/v1/a/c2/one.txt').get_response(self.test_staticweb)
@@ -356,6 +398,19 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(len(json.loads(resp.body)),
                           int(resp.headers['x-container-object-count']))
 
+    def test_container2json_web_mode_explicitly_off(self):
+        resp = Request.blank('/v1/a/c2?format=json',
+            headers={'x-web-mode': 'false'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(resp.content_type, 'application/json')
+        self.assertEquals(len(json.loads(resp.body)),
+                          int(resp.headers['x-container-object-count']))
+
+    def test_container2json_web_mode_explicitly_on(self):
+        resp = Request.blank('/v1/a/c2?format=json',
+            headers={'x-web-mode': 'true'}).get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 404)
+
     def test_container3(self):
         resp = Request.blank('/v1/a/c3').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 301)
@@ -417,6 +472,7 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank('/v1/a/c4/').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 200)
         self.assert_('Listing of /v1/a/c4/' in resp.body)
+        self.assert_('href="listing.css"' in resp.body)
 
     def test_container4indexhtmlauthed(self):
         resp = Request.blank('/v1/a/c4').get_response(self.test_staticweb)
@@ -479,7 +535,9 @@ class TestStaticWeb(unittest.TestCase):
         self.assert_('Listing of /v1/a/c4/subdir/' in resp.body)
         self.assert_('</style>' not in resp.body)
         self.assert_('<link' in resp.body)
-        self.assert_('listing.css' in resp.body)
+        self.assert_('href="../listing.css"' in resp.body)
+        self.assertEquals(resp.headers['content-type'],
+                          'text/html; charset=UTF-8')
 
     def test_container4onetxt(self):
         resp = Request.blank(
@@ -510,6 +568,40 @@ class TestStaticWeb(unittest.TestCase):
         resp = Request.blank('/v1/a/c7/').get_response(self.test_staticweb)
         self.assertEquals(resp.status_int, 404)
 
+    def test_container8listingcss(self):
+        resp = Request.blank(
+                '/v1/a/c8/').get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assert_('Listing of /v1/a/c8/' in resp.body)
+        self.assert_('<link' in resp.body)
+        self.assert_(
+                'href="http://localhost/stylesheets/listing.css"' in resp.body)
+
+    def test_container8subdirlistingcss(self):
+        resp = Request.blank(
+                '/v1/a/c8/subdir/').get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assert_('Listing of /v1/a/c8/subdir/' in resp.body)
+        self.assert_('<link' in resp.body)
+        self.assert_(
+                'href="http://localhost/stylesheets/listing.css"' in resp.body)
+
+    def test_container9listingcss(self):
+        resp = Request.blank(
+                '/v1/a/c9/').get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assert_('Listing of /v1/a/c9/' in resp.body)
+        self.assert_('<link' in resp.body)
+        self.assert_('href="/absolute/listing.css"' in resp.body)
+
+    def test_container9subdirlistingcss(self):
+        resp = Request.blank(
+                '/v1/a/c9/subdir/').get_response(self.test_staticweb)
+        self.assertEquals(resp.status_int, 200)
+        self.assert_('Listing of /v1/a/c9/subdir/' in resp.body)
+        self.assert_('<link' in resp.body)
+        self.assert_('href="/absolute/listing.css"' in resp.body)
+
     def test_subrequest_once_if_possible(self):
         resp = Request.blank(
                 '/v1/a/c4/one.txt').get_response(self.test_staticweb)
@@ -517,8 +609,6 @@ class TestStaticWeb(unittest.TestCase):
         self.assertEquals(resp.headers['x-object-meta-test'], 'value')
         self.assertEquals(resp.body, '1')
         self.assertEquals(self.app.calls, 1)
-        
-
 
 if __name__ == '__main__':
     unittest.main()
